﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.Persistence.Default.Xml;
using HeuristicLab.Persistence.Interfaces;

namespace HeuristicLab.Persistence.GUI {
  public partial class PersistenceConfigurationForm : Form {    
    public PersistenceConfigurationForm() {
      InitializeComponent();
      decomposerList.Items.Clear();            
      foreach ( IDecomposer decomposer in ConfigurationService.Instance.AllDecomposers ) {
        var item = decomposerList.Items.Add(decomposer.GetType().Name);
        item.Checked = true;
        item.Tag = decomposer;
      }      
    }

    private void decomposerList_ItemDrag(object sender, ItemDragEventArgs e) {
      decomposerList.DoDragDrop(decomposerList.SelectedItems, DragDropEffects.Move);
    }

    private void decomposerList_DragEnter(object sender, DragEventArgs e) {      
      if ( e.Data.GetDataPresent(typeof(ListView.SelectedListViewItemCollection).FullName)) {
        e.Effect = DragDropEffects.Move;
      } 
    }

    private void decomposerList_DragDrop(object sender, DragEventArgs e) {
      if (decomposerList.SelectedItems.Count == 0) {
        return;
      }
      //Returns the location of the mouse pointer in the ListView control.
      Point cp = decomposerList.PointToClient(new Point(e.X, e.Y));
      //Obtain the item that is located at the specified location of the mouse pointer.
      ListViewItem dragToItem = decomposerList.GetItemAt(cp.X, cp.Y);
      if (dragToItem == null) {
        return;
      }
      //Obtain the index of the item at the mouse pointer.
      int dragIndex = dragToItem.Index;
      ListViewItem[] sel = new ListViewItem[decomposerList.SelectedItems.Count];
      for (int i = 0; i <= decomposerList.SelectedItems.Count - 1; i++) {
        sel[i] = decomposerList.SelectedItems[i];
      }
      for (int i = 0; i < sel.GetLength(0); i++) {
        //Obtain the ListViewItem to be dragged to the target location.
        ListViewItem dragItem = sel[i];
        int itemIndex = dragIndex;
        if (itemIndex == dragItem.Index) {
          return;
        }
        if (dragItem.Index < itemIndex)
          itemIndex++;
        else
          itemIndex = dragIndex + i;
        //Insert the item at the mouse pointer.
        ListViewItem insertItem = (ListViewItem) dragItem.Clone();
        decomposerList.Items.Insert(itemIndex, insertItem);
        //Removes the item from the initial location while 
        //the item is moved to the new location.
        decomposerList.Items.Remove(dragItem);
      }
      Update();
    }

    private void decomposerList_Resize(object sender, EventArgs e) {
      DecomposersColumn.Width = decomposerList.Width - 4;
    }

    private IEnumerable<IDecomposer> GetDecomposers() {
      List<IDecomposer> decomposers = new List<IDecomposer>();
      foreach ( ListViewItem item in decomposerList.Items ) {
        if (item != null && item.Checked)
          decomposers.Add((IDecomposer) item.Tag);
      }      
      return decomposers;
    }

    private void Update() {
      checkbox.Items.Clear();
      foreach (var decomposer in GetDecomposers())
        checkbox.Items.Add(decomposer.GetType().Name);      
    }

    private void decomposerList_ItemChecked(object sender, ItemCheckedEventArgs e) {
      Update();
    }

  }
}
